use super::operator::UnaryOperator;
use super::{
	token::{self, TokenType},
	ExpressionNode, Parser, Result,
};
use itertools::PeekingNext;
use std::fmt::Debug;

pub struct UnaryExpr {
	pub operator: UnaryOperator,
	pub right: Box<ExpressionNode>,
}

impl UnaryExpr {
	pub fn new(operator: UnaryOperator, right: Box<ExpressionNode>) -> Self {
		Self { operator, right }
	}
}

impl Debug for UnaryExpr {
	fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
		write!(f, "({} {:?})", self.operator, self.right)
	}
}

impl<'a, T: Iterator<Item = token::Token<'a>>> Parser<'a, T> {
	pub(super) fn unary(&mut self) -> Result<ExpressionNode> {
		if let Some(op) = self
			.token_iter
			.peeking_next(|t| matches!(t.token_type, TokenType::Bang | TokenType::Minus))
		{
			let right = Box::new(self.unary()?);
			Ok(ExpressionNode::UnaryExpr(UnaryExpr::new(
				op.token_type.try_into().unwrap(),
				right,
			)))
		} else {
			self.primary()
		}
	}
}
