use crate::lexer::token;

use super::parser::{InnerASTParsingError, Parser, Result};

mod expression_statement;
mod print_statement;
mod variable_assignment_statement;

pub use expression_statement::ExpressionStatement;
pub use print_statement::PrintStatement;
pub use variable_assignment_statement::VariableAssignmentStatement;

use from_variants::FromVariants;

#[derive(FromVariants, Debug)]
pub enum Statement {
	Expression(ExpressionStatement),
	Print(PrintStatement),
	VariableAssignment(VariableAssignmentStatement),
}

macro_rules! all_variants {
	($expr:expr, $val_name:ident => $expr_arm:expr) => {
		{
		use match_any::match_any;
		use $crate::ast::statement::*;
		match_any!($expr, Statement::Expression($val_name) | Statement::Print($val_name) | Statement::VariableAssignment($val_name) => $expr_arm)
		}
	};
}
pub(crate) use all_variants;

impl<'a, T: Iterator<Item = token::Token<'a>>> Parser<'a, T> {
	pub fn statement(&mut self) -> Result<Statement> {
		self.print_statement()
	}
}
