use itertools::PeekingNext;

use super::{InnerASTParsingError, Parser, Result, Statement};
use crate::ast::expression::ExpressionNode;
use crate::lexer::token::{self, Token};

#[derive(Debug)]
pub struct PrintStatement(pub ExpressionNode);
impl PrintStatement {
	pub fn new(expr: ExpressionNode) -> Self {
		Self(expr)
	}
}

impl<'a, T: Iterator<Item = token::Token<'a>>> Parser<'a, T> {
	pub(super) fn print_statement(&mut self) -> Result<Statement> {
		if let Some(Token { location: loc, .. }) = self
			.token_iter
			.peeking_next(|t| matches!(t.token_type, token::TokenType::Print))
		{
			let expr = self.expression()?;
			if let token::TokenType::Semicolon = self.token_iter.peek().token_type {
				self.token_iter.next();
				Ok(PrintStatement::new(expr).into())
			} else {
				Err(loc.wrap(InnerASTParsingError::ExpectedSemi))
			}
		} else {
			self.variable_assignment_statement()
		}
	}
}
