use super::Statement;
use super::{InnerASTParsingError, Parser, Result};
use crate::ast::expression::ExpressionNode;
use crate::lexer::token::{self, TokenType};

#[derive(Debug)]
pub struct VariableAssignmentStatement {
	pub var_name: String,
	pub node: ExpressionNode,
}

impl VariableAssignmentStatement {
	pub fn new(var_name: String, node: ExpressionNode) -> Self {
		Self { var_name, node }
	}
}

impl<'a, T: Iterator<Item = token::Token<'a>>> Parser<'a, T> {
	pub(super) fn variable_assignment_statement(&mut self) -> Result<Statement> {
		if matches!(
			self.token_iter.peek_nth(0).token_type,
			TokenType::Identifier(_)
		) && matches!(self.token_iter.peek_nth(1).token_type, TokenType::Equal)
		{
			let ident = if let TokenType::Identifier(ident) = self.token_iter.next().token_type {
				ident
			} else {
				unreachable!()
			};
			self.token_iter.next();
			let expr = self.expression()?;

			let token = self.token_iter.peek();
			if let token::TokenType::Semicolon = token.token_type {
				self.token_iter.next();
				Ok(VariableAssignmentStatement::new(ident, expr).into())
			} else {
				Err(token.location.wrap(InnerASTParsingError::ExpectedSemi))
			}
		} else {
			self.expression_statement()
		}
	}
}
